@@ -1,14 +1,13 @@
 import asyncio
 import dataclasses
 import json
-import math
-import struct
 import time
 from abc import ABC, abstractmethod
 from collections.abc import Callable, Coroutine
 from typing import Any, TypeVar, final
 
 from roborock import (
+    AppInitStatus,
     DeviceProp,
     DockSummary,
     RoborockCommand,
@@ -46,7 +45,7 @@ from roborock.containers import (
     ValleyElectricityTimer,
     WashTowelMode,
 )
-from roborock.protocol import Utils
+from roborock.protocols.v1_protocol import MapResponse, SecurityData, create_map_response_decoder
 from roborock.roborock_message import (
     ROBOROCK_DATA_CONSUMABLE_PROTOCOL,
     ROBOROCK_DATA_STATUS_PROTOCOL,
@@ -54,15 +53,15 @@ from roborock.roborock_message import (
     RoborockMessage,
     RoborockMessageProtocol,
 )
-from roborock.util import RepeatableTask, get_next_int, unpack_list
+from roborock.util import RepeatableTask, unpack_list
+
+CUSTOM_COMMANDS = {RoborockCommand.GET_MAP_CALIBRATION}
 
 COMMANDS_SECURED = {
     RoborockCommand.GET_MAP_V1,
     RoborockCommand.GET_MULTI_MAP,
 }
 
-CUSTOM_COMMANDS = {RoborockCommand.GET_MAP_CALIBRATION}
-
 CLOUD_REQUIRED = COMMANDS_SECURED.union(CUSTOM_COMMANDS)
 
 WASH_N_FILL_DOCK = [
@@ -107,9 +106,9 @@ class AttributeCache:
             raise err
         return self._value
 
-    async def async_value(self):
+    async def async_value(self, force: bool = False):
         async with self._mutex:
-            if self._value is None:
+            if self._value is None or force:
                 return await self.task.reset()
             return self._value
 
@@ -151,10 +150,15 @@ class RoborockClientV1(RoborockClient, ABC):
     """Roborock client base class for version 1 devices."""
 
     _listeners: dict[str, ListenerModel] = {}
+    _map_response_decoder: Callable[[RoborockMessage], MapResponse | None] | None = None
 
-    def __init__(self, device_info: DeviceData, endpoint: str):
+    def __init__(self, device_info: DeviceData, security_data: SecurityData | None) -> None:
         """Initializes the Roborock client."""
         super().__init__(device_info)
+        if security_data is not None:
+            self._diagnostic_data.update({"misc_info": security_data.to_diagnostic_data()})
+            self._map_response_decoder = create_map_response_decoder(security_data)
+
         self._status_type: type[Status] = ModelStatus.get(device_info.model, S7MaxVStatus)
         self.cache: dict[CacheableAttribute, AttributeCache] = {
             cacheable_attribute: AttributeCache(attr, self._send_command)
@@ -163,7 +167,6 @@ class RoborockClientV1(RoborockClient, ABC):
         if device_info.device.duid not in self._listeners:
             self._listeners[device_info.device.duid] = ListenerModel({}, self.cache)
         self.listener_model = self._listeners[device_info.device.duid]
-        self._endpoint = endpoint
 
     async def async_release(self) -> None:
         await super().async_release()
@@ -175,7 +178,7 @@ class RoborockClientV1(RoborockClient, ABC):
         return self._status_type
 
     async def get_status(self) -> Status:
-        data = self._status_type.from_dict(await self.cache[CacheableAttribute.status].async_value())
+        data = self._status_type.from_dict(await self.cache[CacheableAttribute.status].async_value(force=True))
         if data is None:
             return self._status_type()
         return data
@@ -225,7 +228,8 @@ class RoborockClientV1(RoborockClient, ABC):
                         final_record.square_meter_area += (
                             rec.square_meter_area if rec.square_meter_area is not None else 0
                         )
-                finally:
+                except Exception:
+                    # Return final record when an exception occurred
                     return final_record
             # There are still a few unknown variables in this.
             begin, end, duration, area = unpack_list(record, 4)
@@ -336,34 +340,13 @@ class RoborockClientV1(RoborockClient, ABC):
             return [ServerTimer(*server_timers)]
         return []
 
-    def _get_payload(
-        self,
-        method: RoborockCommand | str,
-        params: list | dict | int | None = None,
-        secured=False,
-    ):
-        timestamp = math.floor(time.time())
-        request_id = get_next_int(10000, 32767)
-        inner = {
-            "id": request_id,
-            "method": method,
-            "params": params or [],
-        }
-        if secured:
-            inner["security"] = {
-                "endpoint": self._endpoint,
-                "nonce": self._nonce.hex().lower(),
-            }
-        payload = bytes(
-            json.dumps(
-                {
-                    "dps": {"101": json.dumps(inner, separators=(",", ":"))},
-                    "t": timestamp,
-                },
-                separators=(",", ":"),
-            ).encode()
-        )
-        return request_id, timestamp, payload
+    async def load_multi_map(self, map_flag: int) -> None:
+        """Load the map into the vacuum's memory."""
+        await self.send_command(RoborockCommand.LOAD_MULTI_MAP, [map_flag])
+
+    async def get_app_init_status(self) -> AppInitStatus:
+        """Gets the app init status (needed for determining vacuum capabilities)."""
+        return await self.send_command(RoborockCommand.APP_GET_INIT_STATUS, return_type=AppInitStatus)
 
     @abstractmethod
     async def _send_command(
@@ -455,25 +438,26 @@ class RoborockClientV1(RoborockClient, ABC):
                             dps = {data_point_number: data_point}
                             self._logger.debug(f"Got unknown data point {dps}")
                 elif data.payload and protocol == RoborockMessageProtocol.MAP_RESPONSE:
-                    payload = data.payload[0:24]
-                    [endpoint, _, request_id, _] = struct.unpack("<8s8sH6s", payload)
-                    if endpoint.decode().startswith(self._endpoint):
-                        try:
-                            decrypted = Utils.decrypt_cbc(data.payload[24:], self._nonce)
-                        except ValueError as err:
-                            raise RoborockException(f"Failed to decode {data.payload!r} for {data.protocol}") from err
-                        decompressed = Utils.decompress(decrypted)
-                        queue = self._waiting_queue.get(request_id)
-                        if queue:
-                            if isinstance(decompressed, list):
-                                decompressed = decompressed[0]
-                            queue.set_result(decompressed)
-                        else:
-                            self._logger.debug("Received response for unknown request id %s", request_id)
+                    if self._map_response_decoder is not None:
+                        map_response = self._map_response_decoder(data)
+                        if map_response is not None:
+                            queue = self._waiting_queue.get(map_response.request_id)
+                            if queue:
+                                queue.set_result(map_response.data)
+                            else:
+                                self._logger.debug(
+                                    "Received unsolicited map response for request_id %s", map_response.request_id
+                                )
+                elif data.protocol == RoborockMessageProtocol.GENERAL_RESPONSE and data.payload is None:
+                    # Api will often send blank messages with matching sequences, we can ignore these.
+                    continue
                 else:
                     queue = self._waiting_queue.get(data.seq)
                     if queue:
-                        queue.set_result(data.payload)
+                        if data.protocol == RoborockMessageProtocol.HELLO_RESPONSE:
+                            queue.set_result(data)
+                        else:
+                            queue.set_result(data.payload)
                     else:
                         self._logger.debug("Received response for unknown request id %s", data.seq)
         except Exception as ex:
